@@ -2,10 +2,10 @@
 /**
  * The Horde_Compress_tar class allows tar files to be read.
  *
- * $Horde: framework/Compress/Compress/tar.php,v 1.4.12.4 2005/07/03 04:52:41 selsky Exp $
+ * $Horde: framework/Compress/Compress/tar.php,v 1.4.12.10 2007/01/02 13:54:11 jan Exp $
  *
- * Copyright 2002-2005 Michael Cochrane <mike@graftonhall.co.nz>
- * Copyright 2003-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2007 Michael Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -64,14 +64,13 @@ class Horde_Compress_tar extends Horde_Compress {
      *         'type'  --  File type
      * </pre>
      */
-    function &decompress(&$data, $params = array())
+    function decompress(&$data, $params = array())
     {
         $position = 0;
         $return_array = array();
 
         while ($position < strlen($data)) {
             $info = @unpack("a100filename/a8mode/a8uid/a8gid/a12size/a12mtime/a8checksum/Ctypeflag/a100link/a6magic/a2version/a32uname/a32gname/a8devmajor/a8devminor", substr($data, $position));
-
             if (!$info) {
                 return PEAR::raiseError(_("Unable to decompress data."));
             }
@@ -81,19 +80,14 @@ class Horde_Compress_tar extends Horde_Compress {
             $position += ceil(octdec($info['size']) / 512) * 512;
 
             if ($info['filename']) {
-                $file = array();
-
-                $file['size'] = octdec($info['size']);
-                $file['date'] = octdec($info['mtime']);
-                $file['name'] = trim($info['filename']);
-
-                if (array_key_exists($info['typeflag'], $this->_types)) {
-                   $file['type'] = $this->_types[$info['typeflag']];
-                } else {
-                   $file['type'] = '';
-                }
-                $file['data'] = '';
-                $file['attr'] = '';
+                $file = array(
+                    'attr' => null,
+                    'data' => null,
+                    'date' => octdec($info['mtime']),
+                    'name' => trim($info['filename']),
+                    'size' => octdec($info['size']),
+                    'type' => isset($this->_types[$info['typeflag']]) ? $this->_types[$info['typeflag']] : null
+                );
 
                 if (($info['typeflag'] == 0) ||
                     ($info['typeflag'] == 0x30) ||
@@ -101,16 +95,18 @@ class Horde_Compress_tar extends Horde_Compress {
                     /* File or folder. */
                     $file['data'] = $contents;
 
-                    $file['attr']  = ($info['typeflag'] == 0x35) ? 'd' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x400) ? 'r' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x200) ? 'w' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x100) ? 'x' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x040) ? 'r' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x020) ? 'w' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x010) ? 'x' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x004) ? 'r' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x002) ? 'w' : '-';
-                    $file['attr'] .= (hexdec(substr($info['mode'], 4, 3)) & 0x001) ? 'x' : '-';
+                    $mode = hexdec(substr($info['mode'], 4, 3));
+                    $file['attr'] =
+                        (($info['typeflag'] == 0x35) ? 'd' : '-') .
+                        (($mode & 0x400) ? 'r' : '-') .
+                        (($mode & 0x200) ? 'w' : '-') .
+                        (($mode & 0x100) ? 'x' : '-') .
+                        (($mode & 0x040) ? 'r' : '-') .
+                        (($mode & 0x020) ? 'w' : '-') .
+                        (($mode & 0x010) ? 'x' : '-') .
+                        (($mode & 0x004) ? 'r' : '-') .
+                        (($mode & 0x002) ? 'w' : '-') .
+                        (($mode & 0x001) ? 'x' : '-');
                 } else {
                     /* Some other type. */
                 }
