@@ -65,9 +65,9 @@ define('PGP_KEYSERVER_TIMEOUT', 10);
  * This class has been developed with, and is only guaranteed to work with,
  * Version 1.21 or above of GnuPG.
  *
- * $Horde: framework/Crypt/Crypt/pgp.php,v 1.85.2.15 2006/07/20 09:02:38 jan Exp $
+ * $Horde: framework/Crypt/Crypt/pgp.php,v 1.85.2.18 2007/03/08 17:49:31 jan Exp $
  *
- * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -355,17 +355,23 @@ class Horde_Crypt_pgp extends Horde_Crypt {
                         $data_array['signature'][$header]['sig_' . $sig_id]['fingerprint'] = $matches[1];
                         $data_array['fingerprint'] = $matches[1];
                     }
+                } elseif (strpos($lowerLine, ':literal data packet:') !== false) {
+                    $header = 'literal';
+                } elseif (strpos($lowerLine, ':encrypted data packet:') !== false) {
+                    $header = 'encrypted';
                 } else {
                     $header = null;
                 }
             } else {
-                if (($header == 'secret_key') || ($header == 'public_key')) {
+                if ($header == 'secret_key' || $header == 'public_key') {
                     if (preg_match("/created\s+(\d+),\s+expires\s+(\d+)/i", $line, $matches)) {
                         $data_array[$header]['created'] = $matches[1];
                         $data_array[$header]['expires'] = $matches[2];
                     } elseif (preg_match("/\s+[sp]key\[0\]:\s+\[(\d+)/i", $line, $matches)) {
                         $data_array[$header]['size'] = $matches[1];
                     }
+                } elseif ($header == 'literal' || $header == 'encrypted') {
+                    $data_array[$header] = true;
                 } elseif ($header) {
                     if (preg_match("/version\s+\d+,\s+created\s+(\d+)/i", $line, $matches)) {
                         $data_array['signature'][$header]['sig_' . $sig_id]['created'] = $matches[1];
@@ -1102,19 +1108,16 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         $good_sig_flag = false;
 
         /* Check for required parameters. */
-        if (!isset($params['pubkey']) ||
-            !isset($params['privkey']) ||
-            !isset($params['passphrase'])) {
+        if (empty($params['no_passphrase']) &&
+            (!isset($params['pubkey']) ||
+             !isset($params['privkey']) ||
+             !isset($params['passphrase']))) {
             return PEAR::raiseError(_("A public PGP key, private PGP key, and passphrase are required to decrypt a message."), 'horde.error');
         }
 
         /* Create temp files. */
         $input = $this->_createTempFile('horde-pgp');
 
-        /* Decryption requires both keyrings. */
-        $pub_keyring = $this->_putInKeyring(array($params['pubkey']));
-        $sec_keyring = $this->_putInKeyring(array($params['privkey']), 'private');
-
         /* Store message in file. */
         $fp = fopen($input, 'w+');
         fputs($fp, $text);
@@ -1124,14 +1127,25 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         $cmdline = array(
             '--always-trust',
             '--armor',
-            '--batch',
-            '--passphrase-fd 0',
-            $sec_keyring,
-            $pub_keyring,
-            '--decrypt',
-            $input
+            '--batch'
         );
-        $result = $this->_callGpg($cmdline, 'w', $params['passphrase'], true, true);
+        if (empty($params['no_passphrase'])) {
+            /* Decryption requires both keyrings. */
+            $pub_keyring = $this->_putInKeyring(array($params['pubkey']));
+            $sec_keyring = $this->_putInKeyring(array($params['privkey']), 'private');
+            $cmdline[] = '--passphrase-fd 0';
+            $cmdline[] = $sec_keyring;
+            $cmdline[] = $pub_keyring;
+        }
+        $cmdline[] = '--decrypt';
+        $cmdline[] = $input;
+
+        /* Decrypt the document now. */
+        if (empty($params['no_passphrase'])) {
+            $result = $this->_callGpg($cmdline, 'w', $params['passphrase'], true, true);
+        } else {
+            $result = $this->_callGpg($cmdline, 'r', null, true, true);
+        }
         if (empty($result->output)) {
             $error = preg_replace('/\n.*/', '', $result->stderr);
             return PEAR::raiseError(_("Could not decrypt PGP data: ") . $error, 'horde.error');
