@@ -47,9 +47,9 @@ define('MIME_STRING', 2);
 /**
  * The MIME:: class provides methods for dealing with MIME standards.
  *
- * $Horde: framework/MIME/MIME.php,v 1.139.4.20 2006/03/24 04:29:52 chuck Exp $
+ * $Horde: framework/MIME/MIME.php,v 1.139.4.23 2007/01/11 23:45:39 jan Exp $
  *
- * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -260,6 +260,9 @@ class MIME {
         $text = '';
         if (is_array($addr_arr)) {
             foreach ($addr_arr as $addr) {
+                if (strlen($text) != 0) {
+                    $text .= ', ';
+                }
                 // Check for groups.
                 if (!empty($addr->groupname)) {
                     $text .= MIME::encode($addr->groupname, $charset) . ': ' . MIME::encodeAddress($addr->addresses) . ';';
@@ -275,9 +278,6 @@ class MIME {
                     }
                     $personal = MIME::encode($addr->personal, $charset);
                 }
-                if (strlen($text) != 0) {
-                    $text .= ', ';
-                }
                 $text .= MIME::trimEmailAddress(MIME::rfc822WriteAddress($addr->mailbox, $addr->host, $personal));
             }
         }
@@ -389,7 +389,7 @@ class MIME {
             $string = str_replace($charset . "'" . $lang . "'", '', $string);
         }
 
-        $lines = preg_split('/' . preg_quote($attribute) . '(?:\*\d)*/', $string);
+        $lines = preg_split('/\s*' . preg_quote($attribute) . '(?:\*\d)*/', $string);
         foreach ($lines as $line) {
             $pos = strpos($line, '*=');
             if ($pos === 0) {
