@@ -4,12 +4,12 @@
  * methods for handling language detection and selection, timezones, and
  * hostname->country lookups.
  *
- * $Horde: framework/NLS/NLS.php,v 1.82.4.13 2006/01/24 07:50:26 selsky Exp $
+ * $Horde: framework/NLS/NLS.php,v 1.82.4.15 2007/01/02 13:54:28 jan Exp $
  *
- * Copyright 1999-2006 Jon Parise <jon@horde.org>
- * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2006 Jan Schneider <jan@horde.org>
- * Copyright 2003-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -35,13 +35,13 @@ class NLS {
         $lang = Util::getFormData('new_lang');
 
         /* First, check if language pref is locked and, if so, set it to its
-           value */
+         * value */
         if (isset($prefs) && $prefs->isLocked('language')) {
             $language = $prefs->getValue('language');
         /* Check if the user selected a language from the login screen */
-        } elseif (!empty($lang)) {
+        } elseif (!empty($lang) && NLS::isValid($lang)) {
             $language = $lang;
-        /* Check if we have a language set in a cookie */
+        /* Check if we have a language set in the session */
         } elseif (isset($_SESSION['horde_language'])) {
             $language = $_SESSION['horde_language'];
         /* Use site-wide default, if one is defined */
