@@ -12,11 +12,11 @@ require_once 'Horde/String.php';
  * wrap    -- Whether to wrap the text or not.
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/html2text.php,v 1.4.10.9 2005/06/10 12:16:19 jan Exp $
+ * $Horde: framework/Text_Filter/Filter/html2text.php,v 1.4.10.14 2007/01/02 13:54:43 jan Exp $
  *
- * Copyright 2003-2005 Jon Abernathy <jon@chuggnutt.com>
+ * Copyright 2003-2007 Jon Abernathy <jon@chuggnutt.com>
  * Original source: http://www.chuggnutt.com/html2text.php
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2007 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -139,7 +139,6 @@ class Text_Filter_html2text extends Text_Filter {
 
             // entities
             '/&nbsp;/i' => ' ',
-            '/&copy;/i' => '(c)',
             '/&trade;/i' => '(tm)',
             '/&#(\d+);/e' => 'String::convertCharset(Text_Filter_html2text::_int2utf8(\'\1\'), "UTF-8", "' . $this->_params['charset'] . '")',
 
@@ -163,6 +162,11 @@ class Text_Filter_html2text extends Text_Filter {
     {
         global $_html2text_state;
 
+        /* Quote block quotes. */
+        $text = preg_replace('/<blockquote type="?cite"?>\n?(.*?)\n?<\/blockquote>\n?/ies',
+                             '"\n\n" . $this->_quote(\'$1\') . "\n\n"',
+                             $text);
+
         /* Strip any other HTML tags. */
         $text = strip_tags($text);
 
@@ -180,12 +184,32 @@ class Text_Filter_html2text extends Text_Filter {
 
         /* Add link list. */
         if (!empty($_html2text_state['linkList'])) {
-            $text .= "\n\n" . _("Links") . ":\n" . str_repeat('-', String::length(_("Links")) + 1) . "\n" . $_html2text_state['linkList'];
+            $text .= "\n\n" . _("Links") . ":\n" .
+                str_repeat('-', String::length(_("Links")) + 1) . "\n" .
+                $_html2text_state['linkList'];
         }
 
         return $text;
     }
 
+    /**
+     * Quotes a chunk of text.
+     *
+     * @param string $text  The text to quote.
+     *
+     * @return string  The quoted text.
+     */
+    function _quote($text)
+    {
+        $text = trim($text);
+        if ($this->_params['wrap']) {
+            $text = wordwrap($text, $this->_params['width'] - 2);
+        }
+        return preg_replace(array('/^/m', '/(\n>\s*$){3,}/m'),
+                            array('> ', "\n>"),
+                            $text);
+    }
+
     /**
      * Returns the UTF-8 character sequence of a Unicode value.
      *
