@@ -2,9 +2,9 @@
 /**
  * VC_svn implementation.
  *
- * Copyright 2000-2005 Anil Madhavapeddy, <anil@recoil.org>
+ * Copyright 2000-2007 Anil Madhavapeddy, <anil@recoil.org>
  *
- * $Horde: framework/VC/VC/svn.php,v 1.28.4.12 2005/09/28 11:45:01 jan Exp $
+ * $Horde: framework/VC/VC/svn.php,v 1.28.4.19 2007/01/02 13:54:47 jan Exp $
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @since   Horde 3.0
@@ -35,7 +35,7 @@ class VC_svn extends VC {
 
     function &queryDir($where)
     {
-        $dir = &new VC_Directory_svn($this, $where);
+        $dir = new VC_Directory_svn($this, $where);
         return $dir;
     }
 
@@ -44,7 +44,7 @@ class VC_svn extends VC {
         return VC_Checkout_svn::get($this, $file->queryFullPath(), $rev);
     }
 
-    function getDiff(&$file, $rev1, $rev2, $type = 'context', $num = 3, $ws = true)
+    function getDiff($file, $rev1, $rev2, $type = 'context', $num = 3, $ws = true)
     {
         return VC_Diff_svn::get($this, $file, $rev1, $rev2, $type, $num, $ws);
     }
@@ -57,7 +57,7 @@ class VC_svn extends VC {
 
     function &getAnnotateObject($filename)
     {
-        $blame = &new VC_Annotate_svn($this, $filename);
+        $blame = new VC_Annotate_svn($this, $filename);
         return $blame;
     }
 
@@ -83,10 +83,10 @@ class VC_Annotate_svn {
     var $SVN;
     var $tmpfile;
 
-    function VC_Annotate_svn(&$rep, $file)
+    function VC_Annotate_svn($rep, $file)
     {
-        $this->SVN = &$rep;
-        $this->file = &$file;
+        $this->SVN = $rep;
+        $this->file = $file;
     }
 
     function doAnnotate($rev)
@@ -114,7 +114,7 @@ class VC_Annotate_svn {
             if (preg_match('/^\s+(\d+)\s+([\w\.]+)\s(.*)$/', $line, $regs)) {
                 $entry = array();
                 $entry['rev']    = $regs[1];
-                $entry['author'] = $regs[2];
+                $entry['author'] = trim($regs[2]);
                 $entry['date']   = '';
                 $entry['line']   = $regs[3];
                 $entry['lineno'] = $lineno++;
@@ -139,13 +139,13 @@ class VC_Annotate_svn {
 class VC_Checkout_svn {
 
     /**
-      * Static function which returns a file pointing to the head of the requested
-      * revision of an RCS file.
-      * @param fullname Fully qualified pathname of the desired RCS file to checkout
-      * @param rev RCS revision number to check out
-      * @return Either a PEAR_Error object, or a stream pointer to the head of the checkout
-      */
-    function get(&$rep, $fullname, $rev)
+     * Static function which returns a file pointing to the head of the requested
+     * revision of an RCS file.
+     * @param fullname Fully qualified pathname of the desired RCS file to checkout
+     * @param rev RCS revision number to check out
+     * @return Either a PEAR_Error object, or a stream pointer to the head of the checkout
+     */
+    function get($rep, $fullname, $rev)
     {
         if (!VC_Revision::valid($rev)) {
             return PEAR::raiseError('Invalid revision number');
@@ -193,7 +193,7 @@ class VC_Diff_svn {
      * @return string|boolean  False on failure, or a string containing the
      *                         diff on success.
      */
-    function get(&$rep, &$file, $rev1, $rev2, $type = 'context', $num = 3,
+    function get($rep, $file, $rev1, $rev2, $type = 'context', $num = 3,
                  $ws = true)
     {
         /* Make sure that the file parameter is valid */
@@ -269,10 +269,10 @@ class VC_Directory_svn {
      * @param string $dn             Path to the directory.
      * @param VC_Directory_svn $pn   The parent VC_Directory to this one.
      */
-    function VC_Directory_svn(&$rep, $dn, $pn = '')
+    function VC_Directory_svn($rep, $dn, $pn = '')
     {
-        $this->rep = &$rep;
-        $this->parent = &$pn;
+        $this->rep = $rep;
+        $this->parent = $pn;
         $this->moduleName = $dn;
         $this->dirName = "/$dn";
         $this->files = array();
@@ -334,11 +334,11 @@ class VC_Directory_svn {
             } elseif (substr($line, -1) == '/') {
                 $this->dirs[] = substr($line, 0, -1);
             } else {
-                $fl = &$this->rep->getFileObject($this->queryDir() . "/$line", $cache, $quicklog);
+                $fl = $this->rep->getFileObject($this->queryDir() . "/$line", $cache, $quicklog);
                 if (is_a($fl, 'PEAR_Error')) {
                     return $fl;
                 } else {
-                    $this->files[] = &$fl;
+                    $this->files[] = $fl;
                 }
             }
         }
@@ -471,9 +471,9 @@ class VC_File_svn extends VC_File {
      *
      * @param string $fl  Full path to this file.
      */
-    function VC_File_svn(&$rep, $fl, $quicklog = false)
+    function VC_File_svn($rep, $fl, $quicklog = false)
     {
-        $this->rep = &$rep;
+        $this->rep = $rep;
         $this->name = basename($fl);
         $this->dir = dirname($fl);
         $this->logs = array();
@@ -484,13 +484,11 @@ class VC_File_svn extends VC_File {
         $this->branches = array();
     }
 
-    function &getFileObject(&$rep, $filename, $cache = null, $quicklog = false)
+    function &getFileObject($rep, $filename, $cache = null, $quicklog = false)
     {
-        /**
-         * The version of the cached data. Increment this whenever the
+        /* The version of the cached data. Increment this whenever the
          * internal storage format changes, such that we must
-         * invalidate prior cached data.
-         */
+         * invalidate prior cached data. */
         $cacheVersion = 2;
         $cacheId = $rep->sourceroot() . '_n' . $filename . '_f' . (int)$quicklog . '_v' . $cacheVersion;
 
@@ -502,7 +500,7 @@ class VC_File_svn extends VC_File {
             $fileOb = unserialize($cache->get($cacheId, 3600));
             $fileOb->setRepository($rep);
         } else {
-            $fileOb = &new VC_File_svn($rep, $filename, $quicklog);
+            $fileOb = new VC_File_svn($rep, $filename, $quicklog);
             $fileOb->setRepository($rep);
             if (is_a(($result = $fileOb->getBrowseInfo()), 'PEAR_Error')) {
                 return $result;
@@ -666,7 +664,7 @@ class VC_File_svn extends VC_File {
 
         fgets($pipe);
         while (!feof($pipe)) {
-            $log = &new VC_Log_svn($this->rep, $this);
+            $log = new VC_Log_svn($this->rep, $this);
             $err = $log->processLog($pipe);
             if ($err) {
                 $rev = $log->queryRevision();
@@ -731,10 +729,10 @@ class VC_Log_svn {
     /**
      * Constructor.
      */
-    function VC_Log_svn($rep, &$fl)
+    function VC_Log_svn($rep, $fl)
     {
         $this->rep = $rep;
-        $this->file = &$fl;
+        $this->file = $fl;
         $this->branches = array();
     }
 
@@ -831,11 +829,9 @@ class VC_Log_svn {
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @package VC
  */
-class VC_Patchset_svn {
+class VC_Patchset_svn extends VC_Patchset {
 
-    var $_rep;
     var $_file;
-    var $_patchsets = array();
 
     /**
      * Create a patchset object.
@@ -847,13 +843,11 @@ class VC_Patchset_svn {
         $this->_file = $file;
     }
 
-    function &getPatchsetObject(&$rep, $filename, $cache = null)
+    function &getPatchsetObject($rep, $filename, $cache = null)
     {
-        /**
-         * The version of the cached data. Increment this whenever the
+        /* The version of the cached data. Increment this whenever the
          * internal storage format changes, such that we must
-         * invalidate prior cached data.
-         */
+         * invalidate prior cached data. */
         $cacheVersion = 1;
         $cacheId = $rep->sourceroot() . '_n' . $filename . '_f_v' . $cacheVersion;
 
@@ -863,20 +857,19 @@ class VC_Patchset_svn {
             // last changed.
             $cache->exists($cacheId, 3600)) {
             $psOb = unserialize($cache->get($cacheId, 3600));
+            $psOb->setRepository($rep);
         } else {
-            $psOb = &new VC_Patchset_svn($filename);
+            $psOb = new VC_Patchset_svn($filename);
+            $psOb->setRepository($rep);
+            if (is_a(($result = $psOb->getPatchsets()), 'PEAR_Error')) {
+                return $result;
+            }
 
             if ($cache) {
                 $cache->set($cacheId, serialize($psOb));
             }
         }
 
-        $psOb->_rep = &$rep;
-
-        if (is_a(($result = $psOb->getPatchsets($rep)), 'PEAR_Error')) {
-            return $result;
-        }
-
         return $psOb;
     }
 
@@ -884,13 +877,11 @@ class VC_Patchset_svn {
      * Populate the object with information about the patchsets that
      * this file is involved in.
      *
-     * @param string $repository  The full repository location.
-     *
      * @return mixed  PEAR_Error object on error, or true on success.
      */
-    function getPatchsets($repository)
+    function getPatchsets()
     {
-        $fileOb = &new VC_File_svn($this->_rep, $this->_file);
+        $fileOb = new VC_File_svn($this->_rep, $this->_file);
         if (is_a(($result = $fileOb->getBrowseInfo()), 'PEAR_Error')) {
             return $result;
         }
