@@ -11,9 +11,9 @@ include_once 'Horde/String.php';
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.34 2006/08/05 13:43:47 karsten Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.45 2007/03/14 15:58:24 jan Exp $
  *
- * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -370,14 +370,14 @@ class Horde_iCalendar {
     }
 
     /**
-     * Locates the first child component of the specified class, and
-     * returns a reference to this component.
+     * Locates the first child component of the specified class, and returns a
+     * reference to it.
      *
      * @param string $type  The type of component to find.
      *
-     * @return mixed (boolean) False if no subcomponent of the specified
-     *                         class exists.
-     *               (Horde_iCalendar_*) A reference to the requested component.
+     * @return boolean|Horde_iCalendar_*  False if no subcomponent of the
+     *                                    specified class exists or a reference
+     *                                    to the requested component.
      */
     function &findComponent($childclass)
     {
@@ -389,7 +389,42 @@ class Horde_iCalendar {
             }
         }
 
-        return false;
+        $component = false;
+        return $component;
+    }
+
+    /**
+     * Locates the first matching child component of the specified class, and
+     * returns a reference to it.
+     *
+     * @param string $childclass  The type of component to find.
+     * @param string $attribute   This attribute must be set in the component
+     *                            for it to match.
+     * @param string $value       Optional value that $attribute must match.
+     *
+     * @return boolean|Horde_iCalendar_*  False if no matching subcomponent of
+     *                                    the specified class exists, or a
+     *                                    reference to the requested component.
+     */
+    function &findComponentByAttribute($childclass, $attribute, $value = null)
+    {
+        $childclass = 'Horde_iCalendar_' . String::lower($childclass);
+        $keys = array_keys($this->_components);
+        foreach ($keys as $key) {
+            if (is_a($this->_components[$key], $childclass)) {
+                $attr = $this->_components[$key]->getAttribute($attribute);
+                if (is_a($attr, 'PEAR_Error')) {
+                    continue;
+                }
+                if ($value !== null && $value != $attr) {
+                    continue;
+                }
+                return $this->_components[$key];
+            }
+        }
+
+        $component = false;
+        return $component;
     }
 
     /**
@@ -503,8 +538,31 @@ class Horde_iCalendar {
         // All subcomponents.
         $matches = null;
         if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/Ui', $vCal, $matches)) {
+            // vTimezone components are processed first. They are
+            // needed to process vEvents that may use a TZID.
             foreach ($matches[0] as $key => $data) {
                 $type = trim($matches[1][$key]);
+                if ($type != 'VTIMEZONE') {
+                    continue;
+                }
+                $component = &Horde_iCalendar::newComponent($type, $this);
+                if ($component === false) {
+                    return PEAR::raiseError("Unable to create object for type $type");
+                }
+                $component->parsevCalendar($data);
+
+                $this->addComponent($component);
+
+                // Remove from the vCalendar data.
+                $vCal = str_replace($data, '', $vCal);
+            }
+
+            // Now process the non-vTimezone components.
+            foreach ($matches[0] as $key => $data) {
+                $type = trim($matches[1][$key]);
+                if ($type == 'VTIMEZONE') {
+                    continue;
+                }
                 $component = &Horde_iCalendar::newComponent($type, $this);
                 if ($component === false) {
                     return PEAR::raiseError("Unable to create object for type $type");
@@ -569,12 +627,15 @@ class Horde_iCalendar {
                     $value = String::convertCharset($value, $charset);
                 }
 
+                // Get timezone info for date fields from $params.
+                $tzid = isset($params['TZID']) ? trim($params['TZID'], '\"') : false;
+
                 switch ($tag) {
                 // Date fields.
                 case 'COMPLETED':
                 case 'CREATED':
                 case 'LAST-MODIFIED':
-                    $this->setAttribute($tag, $this->_parseDateTime($value), $params);
+                    $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                     break;
 
                 case 'BDAY':
@@ -590,14 +651,14 @@ class Horde_iCalendar {
                     if (isset($params['VALUE']) && $params['VALUE'] == 'DATE') {
                         $this->setAttribute($tag, $this->_parseDate($value), $params);
                     } else {
-                        $this->setAttribute($tag, $this->_parseDateTime($value), $params);
+                        $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                     }
                     break;
 
                 case 'TRIGGER':
                     if (isset($params['VALUE'])) {
                         if ($params['VALUE'] == 'DATE-TIME') {
-                            $this->setAttribute($tag, $this->_parseDateTime($value), $params);
+                            $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                         } else {
                             $this->setAttribute($tag, $this->_parseDuration($value), $params);
                         }
@@ -662,16 +723,16 @@ class Horde_iCalendar {
                     $this->setAttribute($tag, trim($value), $params);
                     break;
 
-                // ADR, and N are lists seperated by unescaped
-                // semicolons with a specific number of slots.
+                // ADR, ORG and N are lists seperated by unescaped semicolons
+                // with a specific number of slots.
                 case 'ADR':
                 case 'N':
+                case 'ORG':
                     $value = trim($value);
-                    // As of rfc 2426 2.4.2 semicolon, comma, and
-                    // colon must be escaped (comma is unescaped after
-                    // splitting below).
-                    $value = str_replace(array('\\n', '\\;', '\\:'),
-                                         array($this->_newline, ';', ':'),
+                    // As of rfc 2426 2.4.2 semicolon, comma, and colon must
+                    // be escaped (comma is unescaped after splitting below).
+                    $value = str_replace(array('\\n', '\\N', '\\;', '\\:'),
+                                         array($this->_newline, $this->_newline, ';', ':'),
                                          $value);
 
                     // Split by unescaped semicolons:
@@ -681,29 +742,12 @@ class Horde_iCalendar {
                     $this->setAttribute($tag, trim($value), $params, true, $values);
                     break;
 
-                // ORG is a list seperated by unescaped semicolons but
-                // with no empty values.
-                case 'ORG':
-                    $value = trim($value);
-                    // As of rfc 2426 2.4.2 semicolon, comma, and
-                    // colon must be escaped (comma is unescaped after
-                    // splitting below).
-                    $value = str_replace(array('\\n', '\\;', '\\:'),
-                                         array($this->_newline, ';', ':'),
-                                         $value);
-
-                    // Split by unescaped semicolons:
-                    $values = preg_split('/(?<!\\\\);/', $value, -1, PREG_SPLIT_NO_EMPTY);
-                    $value = str_replace('\\;', ';', $value);
-                    $values = str_replace('\\;', ';', $values);
-                    $this->setAttribute($tag, trim($value), $params, true, $values);
-                    break;
-
                 // String fields.
                 default:
                     if ($this->isOldFormat()) {
-                        // vcalendar 1.0 and vcard 2.1 only escape semicolons
-                        // and use unescaped semicolons to create lists.
+                        // vCalendar 1.0 and vcCard 2.1 only escape
+                        // semicolons and use unescaped semicolons to
+                        // create lists.
                         $value = trim($value);
                         // Split by unescaped semicolons:
                         $values = preg_split('/(?<!\\\\);/', $value);
@@ -715,8 +759,8 @@ class Horde_iCalendar {
                         // As of rfc 2426 2.4.2 semicolon, comma, and
                         // colon must be escaped (comma is unescaped after
                         // splitting below).
-                        $value = str_replace(array('\\n', '\\;', '\\:'),
-                                             array($this->_newline, ';', ':'),
+                        $value = str_replace(array('\\n', '\\N', '\\;', '\\:', '\\\\'),
+                                             array($this->_newline, $this->_newline, ';', ':', '\\'),
                                              $value);
 
                         // Split by unescaped commas:
@@ -763,12 +807,21 @@ class Horde_iCalendar {
             $params = $attribute['params'];
             if ($params) {
                 foreach ($params as $param_name => $param_value) {
-                    // Skip CHARSET for iCalendar 2.0 data, not
-                    // allowed.
+                    /* Skip CHARSET for iCalendar 2.0 data, not allowed. */
                     if ($param_name == 'CHARSET' && !$this->isOldFormat()) {
                         continue;
                     }
-                    $params_str .= ";$param_name=$param_value";
+                    /* Skip VALUE=DATE for vCalendar 1.0 data, not allowed. */
+                    if ($this->isOldFormat() &&
+                         $param_name == 'VALUE' && $param_value == 'DATE') {
+                        continue;
+                    }
+
+                    if ($param_value === null) {
+                        $params_str .= ";$param_name";
+                    } else {
+                        $params_str .= ";$param_name=$param_value";
+                    }
                 }
             }
 
@@ -790,7 +843,7 @@ class Horde_iCalendar {
             case 'RECURRENCE-ID':
                 if (isset($params['VALUE'])) {
                     if ($params['VALUE'] == 'DATE') {
-                        $value = $this->_exportDate($value);
+                        $value = $this->_exportDate($value, $name == 'DTEND' ? '235959' : '000000');
                     } else {
                         $value = $this->_exportDateTime($value);
                     }
@@ -806,7 +859,7 @@ class Horde_iCalendar {
                 foreach ($value as $date) {
                     if (isset($params['VALUE'])) {
                         if ($params['VALUE'] == 'DATE') {
-                            $dates[] = $this->_exportDate($date);
+                            $dates[] = $this->_exportDate($date, '000000');
                         } elseif ($params['VALUE'] == 'PERIOD') {
                             $dates[] = $this->_exportPeriod($date);
                         } else {
@@ -877,7 +930,7 @@ class Horde_iCalendar {
                 // Example of 1.0 style:
                 // ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
                 if ($this->isOldFormat()) {
-                    $value = preg_replace('/MAILTO:/', '', $value);
+                    $value = preg_replace('/MAILTO:/i', '', $value);
                     $value =  $value . ' <' . $value . '>';
                     $params_str = str_replace('PARTSTAT=', 'STATUS=', $params_str);
                     $params_str = str_replace('STATUS=NEEDS-ACTION', 'STATUS=NEEDS ACTION', $params_str);
@@ -888,26 +941,23 @@ class Horde_iCalendar {
                     $params_str = str_replace('RSVP=FALSE', 'RSVP=NO', $params_str);
                 }
                 break;
+
             default:
                 if ($this->isOldFormat()) {
-                    if (is_array($attribute['values'])
-                            && count($attribute['values'])>1) {
+                    if (is_array($attribute['values']) &&
+                        count($attribute['values']) > 1) {
                         $values = $attribute['values'];
-                        if ($name == 'N' || $name == 'ADR') {
+                        if ($name == 'N' || $name == 'ADR' || $name == 'ORG') {
                             $glue = ';';
                         } else {
                             $glue = ',';
                         }
-                        $values  = str_replace(array(';'),
-                                             array('\\;'),
-                                             $values);
+                        $values = str_replace(';', '\\;', $values);
                         $value = implode($glue, $values);
                     } else {
                         /* vcard 2.1 and vcalendar 1.0 escape only
                          * semicolons */
-                        $value = str_replace(array(';'),
-                                             array('\\;'),
-                                             $value);
+                        $value = str_replace(';', '\\;', $value);
                     }
                     if ($name == 'STATUS') {
                         // vcalendar 1.0 has STATUS:NEEDS ACTION while 2.0 has
@@ -918,36 +968,37 @@ class Horde_iCalendar {
                     // or QUOTED-PRINTABLE encoded. Currently we use
                     // QUOTED-PRINTABLE as default.
                     // FIXME: deal with base64 encodings!
-                    if (preg_match("/[^\x20-\x7F]/", $value)
-                            && empty($params['ENCODING']))  {
+                    if (preg_match("/[^\x20-\x7F\r\n]/", $value) &&
+                        empty($params['ENCODING']))  {
                         $params['ENCODING'] = 'QUOTED-PRINTABLE';
                         $params_str .= ';ENCODING=QUOTED-PRINTABLE';
                         // Add CHARSET as well. At least the synthesis client
                         // gets confused otherwise
                         if (empty($params['CHARSET'])) {
-                            $params['CARSET'] = NLS::getCharset();
-                            $params_str .= ';CHARSET=' . $params['CARSET'];
+                            $params['CHARSET'] = NLS::getCharset();
+                            $params_str .= ';CHARSET=' . $params['CHARSET'];
                         }
                     }
                 } else {
-                    if (is_array($attribute['values'])
-                            && count($attribute['values'])>1) {
+                    if (is_array($attribute['values']) &&
+                        count($attribute['values']) > 1) {
                         $values = $attribute['values'];
-                        if ($name == 'N' || $name == 'ADR') {
+                        if ($name == 'N' || $name == 'ADR' || $name == 'ORG') {
                             $glue = ';';
                         } else {
                             $glue = ',';
                         }
-                        $values = str_replace(array(';', ':', ',', 'MAILTO\\:'),
-                                             array('\\;', '\\:', '\\,', 'MAILTO:'),
-                                             $values);
+                        // As of rfc 2426 2.5 semicolon and comma must be
+                        // escaped.
+                        $values = str_replace(array(';', ',', '\\'),
+                                              array('\\;', '\\,', '\\\\'),
+                                              $values);
                         $value = implode($glue, $values);
                     } else {
-                        // As of rfc 2426 2.4.2 semicolon, comma, and colon
-                        // must be escaped. Exclude MAILTO: though. This is a
-                        // hack!
-                        $value = str_replace(array(';', ':', ',', 'MAILTO\\:'),
-                                             array('\\;', '\\:', '\\,', 'MAILTO:'),
+                        // As of rfc 2426 2.5 semicolon and comma must be
+                        // escaped.
+                        $value = str_replace(array(';', ',', '\\'),
+                                             array('\\;', '\\,', '\\\\'),
                                              $value);
                     }
                 }
@@ -955,7 +1006,8 @@ class Horde_iCalendar {
             }
 
             if (!empty($params['ENCODING']) &&
-                $params['ENCODING'] == 'QUOTED-PRINTABLE' && strlen(trim($value)) > 0) {
+                $params['ENCODING'] == 'QUOTED-PRINTABLE' &&
+                strlen(trim($value))) {
                 /* quotedPrintableEncode does not escape CRLFs, but strange
                  * enough single LFs. so convert everything to LF only and
                  * replace afterwards. */
@@ -1042,36 +1094,97 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parses a DateTime field into a unix timestamp.
+     * Grok the TZID and return an offset in seconds from UTC for this
+     * date and time.
+     */
+    function _parseTZID($date, $time, $tzid)
+    {
+        $vtimezone = $this->_container->findComponentByAttribute('vtimezone', 'TZID', $tzid);
+        if (!$vtimezone) {
+            return false;
+        }
+
+        $change_times = array();
+        foreach ($vtimezone->getComponents() as $o) {
+            $t = $vtimezone->parseChild($o, $date['year']);
+            if ($t !== false) {
+                $change_times[] = $t;
+            }
+        }
+
+        if (!$change_times) {
+            return false;
+        }
+
+        sort($change_times);
+
+        // Time is arbitrarily based on UTC for comparison.
+        $t = @gmmktime($time['hour'], $time['minute'], $time['second'],
+                       $date['month'], $date['mday'], $date['year']);
+
+        if ($t < $change_times[0]['time']) {
+            return $change_times[0]['from'];
+        }
+
+        for ($i = 0, $n = count($change_times); $i < $n - 1; $i++) {
+            if (($t >= $change_times[$i]['time']) &&
+                ($t < $change_times[$i + 1]['time'])) {
+                return $change_times[$i]['to'];
+            }
+        }
+
+        if ($t >= $change_times[$n - 1]['time']) {
+            return $change_times[$n - 1]['to'];
+        }
+
+        return false;
+    }
+
+    /**
+     * Parses a DateTime field and returns a unix timestamp. If the
+     * field cannot be parsed then the original text is returned
+     * unmodified.
      *
      * @todo This function should be moved to Horde_Date and made public.
-     * @static
      */
-    function _parseDateTime($text)
+    function _parseDateTime($text, $tzid = false)
     {
         $dateParts = explode('T', $text);
         if (count($dateParts) != 2 && !empty($text)) {
             // Not a datetime field but may be just a date field.
-            if (!$date = Horde_iCalendar::_parseDate($text)) {
-                return $date;
+            if (!preg_match('/^(\d{4})-?(\d{2})-?(\d{2})$/', $text, $match)) {
+                // Or not
+                return $text;
             }
-            return @gmmktime(0, 0, 0, $date['month'], $date['mday'], $date['year']);
+            $newtext = $text.'T000000';
+            $dateParts = explode('T', $newtext);
         }
 
         if (!$date = Horde_iCalendar::_parseDate($dateParts[0])) {
-            return $date;
+            return $text;
         }
         if (!$time = Horde_iCalendar::_parseTime($dateParts[1])) {
-            return $time;
+            return $text;
         }
 
-        if ($time['zone'] == 'UTC') {
-            return @gmmktime($time['hour'], $time['minute'], $time['second'],
-                             $date['month'], $date['mday'], $date['year']);
+        // Get timezone info for date fields from $tzid and container.
+        $tzoffset = ($time['zone'] == 'Local' && $tzid && is_a($this->_container, 'Horde_iCalendar')) ?
+            $this->_parseTZID($date, $time, $tzid) : false;
+        if ($time['zone'] == 'UTC' || $tzoffset !== false) {
+            $result = @gmmktime($time['hour'], $time['minute'], $time['second'],
+                                $date['month'], $date['mday'], $date['year']);
+            if ($tzoffset) {
+                $result -= $tzoffset;
+            }
         } else {
-            return @mktime($time['hour'], $time['minute'], $time['second'],
-                           $date['month'], $date['mday'], $date['year']);
+            // We don't know the timezone so assume local timezone.
+            // FIXME: shouldn't this be based on the user's timezone
+            // preference rather than the server's timezone?
+            $result = @mktime($time['hour'], $time['minute'], $time['second'],
+                              $date['month'], $date['mday'], $date['year']);
         }
+
+        return ($result !== false) ? $result : $text;
     }
 
     /**
@@ -1157,14 +1270,23 @@ class Horde_iCalendar {
     }
 
     /**
-     * Export a Date field.
+     * Exports a date field.
+     *
+     * @param object|array $value  Date object or hash.
+     * @param string $autoconvert  If set, use this as time part to export the
+     *                             date as datetime when exporting to Vcalendar
+     *                             1.0. Examples: '000000' or '235959'
      */
-    function _exportDate($value)
+    function _exportDate($value, $autoconvert = false)
     {
         if (is_object($value)) {
             $value = array('year' => $value->year, 'month' => $value->month, 'mday' => $value->mday);
         }
-        return sprintf('%04d%02d%02d', $value['year'], $value['month'], $value['mday']);
+        if ($autoconvert !== false && $this->isOldFormat()) {
+            return sprintf('%04d%02d%02dT%s', $value['year'], $value['month'], $value['mday'], $autoconvert);
+        } else {
+            return sprintf('%04d%02d%02d', $value['year'], $value['month'], $value['mday']);
+        }
     }
 
     /**
