@@ -2,9 +2,9 @@
 /**
  * Class representing vEvents.
  *
- * $Horde: framework/iCalendar/iCalendar/vevent.php,v 1.31.10.8 2006/01/01 21:28:47 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar/vevent.php,v 1.31.10.11 2007/01/11 18:17:46 jan Exp $
  *
- * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -30,8 +30,9 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
         // Default values.
         $requiredAttributes = array();
         $requiredAttributes['DTSTAMP'] = time();
-        $requiredAttributes['ORGANIZER'] = 'Unknown Organizer';
-        $requiredAttributes['UID'] = $this->_exportDateTime(time()) . '@' . (isset($_SERVER['SERVER_NAME']) ? $_SERVER['SERVER_NAME'] : 'localhost');
+        $requiredAttributes['UID'] = $this->_exportDateTime(time()) .
+            substr(base_convert(microtime(), 10, 36), -16) . '@' .
+            (isset($_SERVER['SERVER_NAME']) ? $_SERVER['SERVER_NAME'] : 'localhost');
 
         $method = !empty($this->_container) ?
             $this->_container->getAttribute('METHOD') : 'PUBLISH';
@@ -87,7 +88,8 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
     function updateAttendee($email, $status, $fullname = '')
     {
         foreach ($this->_attributes as $key => $attribute) {
-            if ($attribute['name'] == 'ATTENDEE' && $attribute['value'] == 'MAILTO:' . $email) {
+            if ($attribute['name'] == 'ATTENDEE' &&
+                $attribute['value'] == 'mailto:' . $email) {
                 $this->_attributes[$key]['params']['PARTSTAT'] = $status;
                 if (!empty($fullname)) {
                     $this->_attributes[$key]['params']['CN'] = $fullname;
@@ -100,7 +102,7 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
         if (!empty($fullname)) {
             $params['CN'] = $fullname;
         }
-        $this->setAttribute('ATTENDEE', 'MAILTO:' . $email, $params);
+        $this->setAttribute('ATTENDEE', 'mailto:' . $email, $params);
     }
 
     /**
@@ -136,7 +138,9 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
             $currentValue = $this->getAttribute($newAttribute['name']);
             if (is_a($currentValue, 'PEAR_error')) {
                 // Already exists so just add it.
-                $this->setAttribute($newAttribute['name'], $newAttribute['value'], $newAttribute['params']);
+                $this->setAttribute($newAttribute['name'],
+                                    $newAttribute['value'],
+                                    $newAttribute['params']);
             } else {
                 // Already exists so locate and modify.
                 $found = false;
@@ -189,7 +193,9 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
             $currentValue = $this->getAttribute($newAttribute['name']);
             if (is_a($currentValue, 'PEAR_error')) {
                 // Already exists so just add it.
-                $this->setAttribute($newAttribute['name'], $newAttribute['value'], $newAttribute['params']);
+                $this->setAttribute($newAttribute['name'],
+                                    $newAttribute['value'],
+                                    $newAttribute['params']);
             } else {
                 // Already exists so locate and modify.
                 $found = false;
