@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: horde/services/obrowser/index.php,v 1.6.10.3 2005/05/12 18:31:09 chuck Exp $
+ * $Horde: horde/services/obrowser/index.php,v 1.6.10.6 2007/01/02 13:55:16 jan Exp $
  *
- * Copyright 2004-2005 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 2004-2007 Charles J. Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -37,7 +37,7 @@ if (!count($list)) {
 }
 
 $tpl = <<<TPL
-<script language="JavaScript" type="text/javascript">
+<script type="text/javascript">
 function chooseObject(oid)
 {
     if (!window.opener || !window.opener.obrowserCallback) {
